@@ -1,4 +1,5 @@
 import os
+import sys
 import json
 import time
 import fire
@@ -35,12 +36,12 @@ DEFAULT_CONFIG = {
         "description": "test"
     },
     "controller_url": "http://127.0.0.1:5917",
-    "host": "127.0.0.1",  # 自身IP，可不填
-    "log_level": "DEBUG"
+    "host": "127.0.0.1",  # 自身IP，用于控制器健康检查
+    "log_level": "DEBUG",
+    "log_file_path": os.getcwd()
 }
 
 
-
 def initialize_logger(log_file_path=os.getcwd(), log_file_name=None, file_log_level="INFO", stderr_log_level="DEBUG"):
     """
     Get logging handler
@@ -59,8 +60,6 @@ def initialize_logger(log_file_path=os.getcwd(), log_file_name=None, file_log_le
     return logger
 
 
-
-
 class Executor(object):
     """Executor class."""
     DEFAULT_POOL_SIZE = {"loop": 1, "async": 100, "thread": 100, "process": 10}
@@ -89,7 +88,6 @@ class Executor(object):
         self.running = True
         self.kafka_arrangement = {}
 
-
     def init_runtimedump(self):
         self.runtime = load_runtime()
         if not self.runtime:
@@ -99,6 +97,7 @@ class Executor(object):
                 "tags": self.config["tags"],
                 'process': {
                     "main": os.getpid(),
+                    "api": "",
                     "triggers": []
                 }
             }
@@ -108,9 +107,7 @@ class Executor(object):
         try:
             with open(self.config_file, "r") as f:
                 self.config = yaml.safe_load(f.read())
-            assert {"type", "tags", "controller_url"} <= set(self.config.keys())
-            if "host" not in self.config.keys():
-                self.config["host"] = "0.0.0.0"
+            assert {"host", "type", "tags", "controller_url"} <= set(self.config.keys())
             if "log_level" not in self.config.keys():
                 self.config["log_level"] = "INFO"
 
@@ -131,10 +128,12 @@ class Executor(object):
         """
         fire.Fire({
             "start": self.start,
+            "stop": self.stop,
+            "restart": self.restart,
             "show_config": self.show_config,
             "init": self.init_config,
             "status": self.status,
-            "config": self.edit_config 
+            "config": self.edit_config
         })
 
     # ----------------- Data Processing ------------------------------------------------
@@ -219,8 +218,6 @@ class Executor(object):
 
         return wrapper
 
-
-
     def update_config(self):
         try:
             reboot = request.json["reboot"]
@@ -235,7 +232,7 @@ class Executor(object):
         return {"state": True, "info": "Success"}, 200
 
     @retry(wait=wait_fixed(3))
-    def register(self, port):
+    def register(self):
         """
         Register request
         :param port: api port
@@ -244,25 +241,24 @@ class Executor(object):
             data = {
                 "type": self.config["type"],
                 "tags": self.config["tags"],
-                "host": self.config["host"] if "host" in self.config.keys() else "0.0.0.0",
-                "port": port,
-                "path": "/health" if self.health_check_options["path"] is None else self.health_check_options["path"]
+                "message": "register"
             }
             if self.name is not None:
                 data["name"] = self.name
 
-            re = requests.post(self.config["controller_url"] + "/executor/register", json=data)
+            re = requests.post(self.config["controller_url"] + "/executor/health", json=data)
+            if re.status_code // 100 != 2:
+                logger.error("Error in register:" + re.text)
+                raise IOError("Error in register")
             response = re.json()
             self.cookie = re.headers["Set-Cookie"]
-            logger.debug(response)
             self.name = response["name"]
             self.kafka_url = response["config"]["kafka_url"]
             self.kafka_topic = response["config"]["kafka_topic"]
             self.kafka_group = response["config"]["kafka_group"]
             if response["segment"]:
                 self.kafka_arrangement = response["segment"]
-            else:
-                self.kafka_arrangement = {}
+                self.queue.put_nowait(response["segment"])
         except Exception as err:
             logger.exception(err)
             raise
@@ -299,7 +295,7 @@ class Executor(object):
             raise
 
     @retry(wait=wait_fixed(3))
-    def finish_report(self, task, output_length, events, has_output,errors):
+    def finish_report(self, task, output_length, events, has_output, errors):
         try:
             data = {
                 "type": "finished",
@@ -322,12 +318,13 @@ class Executor(object):
             raise
 
     @retry(wait=wait_fixed(3))
-    def end_report(self, task, has_output, result, errors):
+    def end_report(self, task, has_output, id, result, errors):
         try:
             data = {
                 "type": "end",
                 "task_id": task.task_id,
                 "branch_id": task.branch_id,
+                "branch_amount": id,
                 "current_position": task.position,
                 "has_output": has_output,
                 "result": result,
@@ -335,10 +332,11 @@ class Executor(object):
             }
             re = requests.put(self.config["controller_url"] + "/task", json=data, headers={"Cookie": self.cookie})
             response = re.json()
+            print(response)
             if response["state"] is False:
                 raise ValueError("Error in controller response: %s" % re.text)
             if "segment" in response.keys() and response["segment"] is not None:
-                self.kafka_arrangement = response["segment"]
+                self.queue.put_nowait(self.kafka_arrangement)
         except Exception as err:
             logger.exception(err)
             raise
@@ -355,7 +353,9 @@ class Executor(object):
         Receive input and task information and execute data processing , then output to kafka
         """
         operation, config = self.receive_report(task.task_id, task.branch_id, task.position, len(data),
-                                                            partition, offset)
+                                               partition, offset)
+
+
         if not operation:
             logger.info("The controller prevents the task from executing, skip!")
             return
@@ -375,8 +375,6 @@ class Executor(object):
             error_collector.add_many("preprocess", errors)
         else:
             preprocessed = data
-        logger.debug("preprocessed:")
-        logger.debug(preprocessed)
 
         if len(preprocessed) == 0:
             logger.warning("No valid preprocessing result!")
@@ -391,8 +389,6 @@ class Executor(object):
                                              pool_size=self.handle_config["pool_size"])
             if processed is None:
                 processed = []
-            logger.debug("processed:")
-            logger.debug(processed)
 
         has_output = True
         if len(processed) == 0:
@@ -400,19 +396,14 @@ class Executor(object):
         else:
             # check end condition
             if "depth" in task.end_condition["builtin"] and task.properties["builtin"]["depth"] + 1 > \
-                    task.end_condition["builtin"][
-                        "depth"]:
+                    task.end_condition["builtin"]["depth"]:
                 has_output = False
             elif "create_at" in task.end_condition["builtin"] and time.time() - task.properties["builtin"][
                 "create_at"] > \
                     task.end_condition["builtin"]["create_at"]:
                 has_output = False
 
-        outputs = self.finish_report(task, len(processed), self.events, has_output,error_collector.export())
-        logger.debug("outputs")
-        logger.debug(outputs)
-
-
+        outputs = self.finish_report(task, len(processed), self.events, has_output, error_collector.export())
 
         # check externals
         # [FIX ME]
@@ -429,8 +420,6 @@ class Executor(object):
                                                           func=self.afterprocess_func[output["output_type"]][0],
                                                           timeout=self.afterprocess_func[output["output_type"]][1],
                                                           mode="total")
-                    logger.debug("afterprocessed[%s]" % output["output_type"])
-                    logger.debug(afterprocessed)
                     error_collector.add_many("afterprocess", errors)
                 else:
                     afterprocessed = processed
@@ -446,7 +435,7 @@ class Executor(object):
                             "properties": task.properties,
                             "tags": output["tags"],
                             "type": output["output_type"],
-                            "current_position":output["position"],
+                            "current_position": output["position"],
                             "end_conditions": task.end_condition
                         },
                         "body": data
@@ -461,14 +450,13 @@ class Executor(object):
             result = True
             info = ""
 
-        self.end_report(task, has_output, result, error_collector.export())
+        self.end_report(task, has_output, id, result, error_collector.export())
 
     # ---------------------------  Process ------------------------------------------------
     def inside_kafka_process(self):
         """
         Process, listen to the inside kafka message, and handle it
         """
-        logger.debug(id(self))
         self.kafka_monitor = KafkaMonitor(self)
         while True:
             try:
@@ -489,8 +477,10 @@ class Executor(object):
 
             try:
                 headers = message["headers"]
-                task = Task(task_id=headers["task_id"], branch_id=headers["branch_id"], position=headers["current_position"],
-                            data_type=headers["type"] ,properties=headers["properties"], end_condition=headers["end_conditions"])
+                task = Task(task_id=headers["task_id"], branch_id=headers["branch_id"],
+                            position=headers["current_position"],
+                            data_type=headers["type"], properties=headers["properties"],
+                            end_condition=headers["end_conditions"])
                 self.data_process(task, message["body"], partition=msg.partition(), offset=msg.offset())
             except Exception as err:
                 logger.exception(err)
@@ -499,49 +489,6 @@ class Executor(object):
         self.running = False
 
 
-    def init_api_process(self, port, queue):
-        self.app = Flask(__name__)
-        self.queue = queue
-        @self.app.route(self.health_check_options['path'] or "/health", methods=['GET'])
-        def health_check_function():
-            headers = {
-                "Content-Type": "application/json",
-                "whoami": self.name
-            }
-            if self.health_check_options["action"] is None:
-                return json.dumps({"state": True, "info": "Default health check response.."}), 200, headers
-            else:
-                try:
-                    resp = self.health_check_options["action"]()
-                    if type(resp) == str:
-                        headers.pop("Content-Type")
-                        return resp, 200, headers
-                    elif type(resp) == dict:
-                        return jsonify(resp), 200, headers
-                    else:
-                        logger.warning("Unsupported return type: %s!" % (str(type(resp))))
-                        raise TypeError("Unsupported return type!")
-                except:
-                    logger.warning("Error in health check response:\n", exc_info=True)
-                    return json.dumps({"state": True, "info": "Error in health check response"}), 200, headers
-
-        @self.app.route("/segment", methods=["PUT"])
-        def update_kafka():
-            try:
-                segment = request.json["segment"]
-            except:
-                logger.error("Error in parse params:\n", exc_info=True)
-                return jsonify({"state": False, "info": "error in parsing params"})
-            self.kafka_arrangement = segment
-            self.queue.put_nowait(segment)
-            logger.info("update kafka segments!")
-            logger.debug(self.kafka_arrangement)
-
-            return jsonify({"state": True, "info": "Success"})
-
-        self.app.run(host="0.0.0.0", port=port, debug=False)
-
-
     # --------------------------  Trigger  --------------------------------------
     def add_trigger(self, trigger):
         """
@@ -595,8 +542,8 @@ class Executor(object):
                 continue
             if trigger["type"] == "time":
                 process = Process("time_trigger", action=time_process, args=(
-                copy.deepcopy(self.triggers[trigger["name"]]),
-                trigger["interval"] if "interval" in trigger else self.triggers[trigger["name"]].interval, task))
+                    copy.deepcopy(self.triggers[trigger["name"]]),
+                    trigger["interval"] if "interval" in trigger else self.triggers[trigger["name"]].interval, task))
                 process.start()
                 self.runtime["process"]["trigger"].append({"task": task, "name": trigger["name"],
                                                            "interval": trigger["interval"] if "interval" in trigger else
@@ -659,20 +606,31 @@ class Executor(object):
                 break
 
     # -------------------------  Others  -------------------------------------
-    def health_check(self, path=None):
-        '''
-        The function for handling data which is defined by user
-        :param name(optional){string}: the name of the function of handling outputs
-        '''
+    def health_report(self, queue, interval = 5):
+        s = requests.Session()
+        assignment = self.kafka_arrangement
+        while True:
+            try:
+                resp = s.post(url = self.config["controller_url"] + "/executor/health",
+                              json = {"message": "normal"},
+                              headers={"Cookie": self.cookie})
+                assert resp.status_code // 100 == 2
+                
+                data = resp.json()
+                print(data)
+                if data["segment"] != assignment:
+                    print(assignment)
+                    assignment  = data["segment"]
+                    queue.put_nowait(data["segment"])
+            except AssertionError:
+                logger.error("Error in health check, code: %d, Info: %s" % (resp.status_code, resp.text))
+            except Exception as err:
+                logger.error("Error in health check, Error %s" % err.__repr__())
+            time.sleep(interval)
+        s.close()
+
 
-        def wrapper(func):
-            self.health_check_options = {
-                "action": func,
-                "path": path
-            }
-            self.reset_cmd()
 
-        return wrapper
 
 
 
@@ -683,8 +641,7 @@ class Executor(object):
         # Reload the config
         self.reload_config()
         logger.debug(self.config)
-        initialize_logger(stderr_log_level=self.config["log_level"].upper())
-        
+        initialize_logger(log_file_path=self.config["log_file_path"], stderr_log_level=self.config["log_level"].upper())
 
         # Initialize RuntimeDump Object
         self.init_runtimedump()
@@ -700,14 +657,13 @@ class Executor(object):
 
         # ---------- Register -----------
 
-        self.register(port)
+        self.queue = Queue(10)
+        self.register()
         self.runtime["name"] = self.name
-        write_runtime(self.runtime)
 
         # ---------- Start API Process -----------
 
-        self.queue = Queue(10)
-        self.api_process  = Process("api", action=self.init_api_process, args=(port, self.queue))
+        self.api_process = Process("api", action=self.health_report, args=(self.queue,))
         self.api_process.start()
         self.runtime["process"]["api"] = self.api_process.pid
 
@@ -718,6 +674,8 @@ class Executor(object):
         # ---------- Main Process set daemon
         self.ready = True
         self.runtime["process"]["main"] = os.getpid()
+
+        write_runtime(self.runtime)
         self.inside_kafka_process()
 
     def restart(self):
@@ -743,7 +701,7 @@ class Executor(object):
         os.kill(api_pid, signal.SIGKILL)
         main_pid = self.runtime["process"]["main"]
         os.kill(main_pid, signal.SIGKILL)
-        
+
         cprint(pyfiglet.figlet_format('Autumn', font='graffiti'), 'green', attrs=['blink', 'bold'])
         logger.info("stop")
 
@@ -773,18 +731,18 @@ class Executor(object):
         print(colored('   - ', 'red', attrs=["bold"]), end="")
         print(Fore.GREEN + "Status: ", end="")
 
-        if (is_alive(self.runtime.get("main_pid"))):
+        if (is_alive(self.runtime["process"]["main"])):
             print(emoji.emojize(':rocket:') + "  running...\n")
         else:
             print(emoji.emojize(':construction:') + "  not active.\n")
 
-        print(Fore.GREEN + "HealthCheck:", end="\n")
+        print(Fore.GREEN + "API:", end="\n")
         print(colored('   - ', 'red', attrs=["bold"]), end="")
         print(Fore.GREEN + "PID: ", end="")
         print(Fore.WHITE + "%s  " % (self.runtime.get("healthcheck_pid")))
         print(colored('   - ', 'red', attrs=["bold"]), end="")
         print(Fore.GREEN + "Status: ", end="")
-        if (is_alive(self.runtime.get("healthcheck_pid"))):
+        if (self.runtime["process"]["api"] and is_alive(self.runtime["process"]["api"])):
             print(emoji.emojize(':rocket:') + "  running...\n")
         else:
             print(emoji.emojize(':construction:') + "  not active.\n")
@@ -815,6 +773,8 @@ class Executor(object):
         if action == "set":
             with open(self.config_file, "r") as f:
                 self.config = yaml.safe_load(f.read())
-            self.config[param1]=param2
+            if param1 == "tags":
+                param2 = param2.split(",")
+            self.config[param1] = param2
             with open(self.config_file, "w+") as f:
                 yaml.dump(self.config, f, default_flow_style=False, Dumper=yaml.SafeDumper)
